<?php

include 'settings.php';

//If the function doesn't exist
if (!function_exists("bcadd")) {
	function bcadd($val1, $val2) {
		return (double)$val1 + (double)$val2;
	}
	function bcsub($val1, $val2) {
		return (double)$val1 - (double)$val2;
	}
}

// Ensure bc functions return to 2 decimal places
bcscale(2);

if ($apiKey == "") {
	echo('{"messages": {"message": "API Key not set."}, "result": "ERROR"}');
	exit;
}

if (!isset($_REQUEST["passphrase"]) || $_REQUEST["passphrase"] != $passphrase || !isset($_REQUEST["store_domain"]) || $_REQUEST["store_domain"] == "") {
	echo('{"messages": {"message": "Incorrect store domain or passphrase"}, "result": "ERROR"}');
	exit;
}

if (!file_exists($export_folder) || !is_dir($export_folder)) {
	echo('{"messages": {"message": "Can\'t find the export folder. Make sure it exists."}, "result": "ERROR", "export": "'.$export_folder.'"}');
	exit;
} elseif (!is_writable($export_folder)) {
	echo('{"messages": {"message": "Export folder is not writeable, please check your folder permissions."}, "result": "ERROR"}');
	exit;
}

// Defaults
$seperator = ",";
$filetype = ".csv";
$include_line_items = true;
$first_run = true;
$store_version = "1.0";
$transaction_headers = array("transaction_id", "store_id", "transaction_date", "product_total", "tax_total", "shipping_total", "discount_total", "order_total", "purchase_order", "cc_type", "cc_number_masked", "paypal_payer_id", "processor_response", "customer_id", "is_anonymous", "customer_first_name", "customer_last_name", "customer_company", "customer_address1", "customer_address2", "customer_city", "customer_state", "customer_postal_code", "customer_country", "customer_phone", "customer_email", "customer_ip", "custom_fields", "coupons_used", "shipping_service_description", "shipping_first_name", "shipping_last_name", "shipping_company", "shipping_address1", "shipping_address2", "shipping_city", "shipping_state", "shipping_postal_code", "shipping_country", "shipping_phone");
$transaction_headers = array_merge($transaction_headers, $extra_custom_fields_columns);
$product_headers = array("category_code", "product_name", "product_code", "product_price", "product_quantity", "product_weight", "product_options", "sub_token_url");
$product_headers = array_merge($product_headers, $extra_product_options_columns);
$subscription_headers = array("subscription_frequency", "subscription_startdate", "subscription_nextdate", "subscription_enddate", "is_future_line_item");
$export_body = "";

if (isset($_REQUEST["filetype"])) {
	if ($_REQUEST["filetype"] == "tab") {
		$filetype = ".txt";
		$seperator = "\t";
	}
}
if (isset($_REQUEST["includelineitems"])) {
	if ($_REQUEST["includelineitems"] == 0) {
		$include_line_items = false;
	}
}
if (isset($_REQUEST["store_version"])) {
	$store_version = $_REQUEST["store_version"];
}

$storeDomain = $_REQUEST["store_domain"];
$export_name = str_replace(".", "-", $storeDomain);

if ($include_line_items) {
	$export_name .= "_with-line-items";
}

$filter_data = array();
$filter_data["api_token"] = $apiKey;
$filter_data["api_action"] = "transaction_list";
$filter_data["status_filter"] = "all";

if (isset($_REQUEST["pagination_start"])) {
	if ((float)$_REQUEST["pagination_start"] > 0) {
		$first_run = false;
	}
	$filter_data["pagination_start"] = $_REQUEST["pagination_start"];
} else {
	echo('{"messages": {"message": "Incorrect pagination start value"}, "result": "ERROR"}');
	exit;
}

if (isset($_REQUEST["entries_per_page"])) {
	$filter_data["entries_per_page"] = $_REQUEST["entries_per_page"];
} else {
	$filter_data["entries_per_page"] = 30;
}

$request_type = "live-and-test";
if (isset($_REQUEST["is_test_filter"])) {
	$filter_data["is_test_filter"] = $_REQUEST["is_test_filter"];
	if ($_REQUEST["is_test_filter"] != "") {
		$request_type = ((int)$_REQUEST["is_test_filter"] == 0) ? "live" : "test";
	}
}
$export_name .= "_" . $request_type;

$display_type = "visible-and-hidden";
if (isset($_REQUEST["hide_transaction_filter"])) {
	$filter_data["hide_transaction_filter"] = $_REQUEST["hide_transaction_filter"];
	if ($_REQUEST["hide_transaction_filter"] != "") {
		$display_type = ((int)$_REQUEST["hide_transaction_filter"] == 0) ? "visible" : "hidden";
	}
}
$export_name .= "_" . $display_type;

if (isset($_REQUEST["date_filter"])) {
	$date_filter = array();
	if (isset($_REQUEST["transaction_date_filter_begin"])) {
		$filter_data["transaction_date_filter_begin"] = $_REQUEST["transaction_date_filter_begin"];
		$date_filter[] = $_REQUEST["transaction_date_filter_begin"];
	}
	if (isset($_REQUEST["transaction_date_filter_end"])) {
		$filter_data["transaction_date_filter_end"] = $_REQUEST["transaction_date_filter_end"];
		$date_filter[] = $_REQUEST["transaction_date_filter_end"];
	}
	$export_name .= "_" . implode("-", $date_filter);
}

$file_name = $export_name . $filetype;

// Fetch the transactions
$results = array();
$response = callAPI($filter_data);

if ($response == false) {
	// Something went wrong...
	return false;
}

$foxyResponse = simplexml_load_string($response, NULL, LIBXML_NOCDATA);

if ((string)$foxyResponse->result == "ERROR") {
	// API returned an error, return it
	echo json_encode($foxyResponse);
	exit();
}

foreach($foxyResponse->transactions->transaction as $transaction) {
	$results[] = $transaction;
}


///////////////////////////////////////////////
// Process the transactions

foreach($results as $result) {

	//This is for testing
	if ((string)$result->id == "2750040") {
		//print_r($result);
	}

	// Only ouptut completed transactions
	if ((string)$result->status != "" && in_array((string)$result->status, array("", "captured", "approved", "authorized")) == false) break;

	$transaction_id = $result->id;

	$transaction_row = true;
	foreach($result->transaction_details->transaction_detail as $product) {

		$export_body_row = array();

		if (!$include_line_items && !$transaction_row) {
			// Only export the first row if not including line items
			break;
		}

		if ($transaction_row) {
			// This is the first row, add in the transaction information
			foreach($transaction_headers as $header) {
				if (!in_array($header, array("discount_total", "custom_fields", "coupons_used")) && !in_array($header, $extra_custom_fields_columns)) {
					if ($header == "transaction_id") {
						$header = "id";
					}
					if (isset($result->{$header})) {
						$export_body_row[] = cell((string)$result->{$header});
					} else {
						$export_body_row[] = '""';
					}
				} else {
					// Special columns
					if ($header == "discount_total") {
						$total_discounts = 0;
						foreach($result->discounts->discount as $discount) {
							$total_discounts = bcadd($total_discounts, (float)$discount->amount);
						}
						$export_body_row[] = cell($total_discounts);
						unset($discount, $total_discounts);

					} elseif ($header == "coupons_used") {
						$coupons_used = array();
						foreach($result->discounts->discount as $discount) {
							$coupons_used[] = (string)$discount->code . ":" . (string)$discount->amount;
						}
						$export_body_row[] = cell(implode(" - ", $coupons_used));
						unset($discount, $total_discounts);

					} elseif ($header == "custom_fields") {
						$custom_fields_used = array();
						foreach($result->custom_fields->custom_field as $custom_field) {
							$custom_fields_used[] = (string)$custom_field->custom_field_name . ":" . (string)$custom_field->custom_field_value;
						}
						$export_body_row[] = cell(implode(" - ", $custom_fields_used));
						unset($custom_field, $custom_fields_used);

					} elseif (in_array($header, $extra_custom_fields_columns)) {
						$custom_field_value = "";
						foreach($result->custom_fields->custom_field as $custom_field) {
							if ($header == (string)$custom_field->custom_field_name) {
								$custom_field_value = (string)$custom_field->custom_field_value;
							}
						}
						$export_body_row[] = $custom_field_value;
					}
				}
			}
		} else {
			// This is a subsequent row, add empty columns for transaction columns
			$export_body_row[] = cell($transaction_id);
			$export_body_row = array_merge($export_body_row, array_fill(1, bcsub(count($transaction_headers), 1), ""));
		}

		if ($include_line_items) {
			$modify_price = 0;
			$modify_weight = 0;
			$product_options = array();
			foreach($product->transaction_detail_options->transaction_detail_option as $product_option) {
				$product_options[] = (string)$product_option->product_option_name . ":" . (string)$product_option->product_option_value;
				$modify_price = bcadd($modify_price, (float)$product_option->price_mod);
				$modify_weight = bcadd($modify_weight, (float)$product_option->weight_mod);
			}
			unset($transaction_detail_options);

			foreach($product_headers as $header) {

				if (!in_array($header, array("product_options", "product_price", "product_weight")) && !in_array($header, $extra_product_options_columns)) {
					if (isset($product->{$header})) {
						$export_body_row[] = cell((string)$product->{$header});
					} else {
						$export_body_row[] = '""';
					}
				} else {
					if ($header == 'product_price') {
						$export_body_row[] = cell(bcadd((string)$product->product_price, $modify_price));
						break;
					} elseif ($header == "product_weight") {
						$export_body_row[] = cell(bcadd((string)$product->product_weight, $modify_weight));
						break;
					} elseif ($header == "product_options") {
						$export_body_row[] = cell(implode(" - ", $product_options));
						break;
					} elseif (in_array($header, $extra_product_options_columns)) {
						$custom_field_value = "";
						foreach($product->transaction_detail_options->transaction_detail_option as $product_option) {
							if ($header == (string)$product_option->product_option_name) {
								$custom_field_value = (string)$product_option->product_option_value;
							}
						}
						$export_body_row[] = $custom_field_value;
					}
				}
			}

			if ($include_subscription_columns) {
				foreach($subscription_headers as $header) {
					if (isset($product->{$header}) && $product->{$header} != "0000-00-00") {
						$export_body_row[] = cell((string)$product->{$header});
					} else {
						$export_body_row[] = '""';
					}
				}
			}
			unset($header, $modify_price, $modify_weight, $product_options);
		}

		$export_body .= "\r\n" . implode($seperator, $export_body_row);

		// Only allow one transaction row per transaction
		$transaction_row = false;

	}
	unset($product);
}

unset($results, $result);

$export_headers = "";

if ($include_line_items) {
	$export_headers = array_merge($transaction_headers, $product_headers);

	if ($include_subscription_columns) {
		$export_headers = array_merge($export_headers, $subscription_headers);
	}
} else {
	$export_headers = $transaction_headers;
}

if ($first_run) {
	$export_body = implode($seperator, $export_headers) . $export_body;
}

///////////////////////////////////////////////
// Write to the file

if ($first_run) {
	$export_file = fopen($export_folder . $file_name, 'w') or die('Cannot open file:  '.$file_name);
	// $export_file2 = fopen($export_folder . "__" . $file_name, 'w') or die('Cannot open file:  '.$file_name);
} else {
	$export_file = fopen($export_folder . $file_name, 'a') or die('Cannot open file:  '.$file_name);
	// $export_file2 = fopen($export_folder . "__" . $file_name, 'a') or die('Cannot open file:  '.$file_name);
}

fwrite($export_file, $export_body);
// fwrite($export_file2, $response);
fclose($export_file);
// fclose($export_file2);
unset($export_file);

$export_size = filesize($export_folder . $file_name);

if ($export_size < 1024) {
    $export_size = $export_size .' B';
} elseif ($export_size < 1048576) {
    $export_size = round($export_size / 1024, 2) .' KB';
} elseif ($export_size < 1073741824) {
    $export_size = round($export_size / 1048576, 2) . ' MB';
}

///////////////////////////////////////////////
// Send a response include the API statistics

if (!isset($foxyResponse->store_version)) {
	$foxyResponse->store_version = $foxyResponse->transactions->transaction[count($foxyResponse->transactions->transaction)-1]->store_version;
}
unset($foxyResponse->transactions);
$foxyResponse->file->name = $file_name;
$foxyResponse->file->size = $export_size;
echo json_encode($foxyResponse);

unset($foxyResponse, $result, $response, $transaction, $product, $header, $export_body_row, $export_body, $export_headers, $transaction_headers, $product_headers);



function cell($cell) {
	return '"' . str_replace('"','""',$cell) . '"';
}

function callAPI($foxyData) {
	global $storeDomain;
	unset($response);
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, "https://" . $storeDomain . "/api");
	curl_setopt($ch, CURLOPT_POSTFIELDS, $foxyData);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
	curl_setopt($ch, CURLOPT_TIMEOUT, 15);
	// If you get SSL errors, you can uncomment the following, or ask your host to add the appropriate CA bundle
	// curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	$response = trim(curl_exec($ch));

	if ($response == false) {
		echo('{"messages": {"message": ' . json_encode("CURL Error: " . curl_error($ch)) . '}, "result": "ERROR"}');
		exit();
	}

	curl_close($ch);
	unset($ch);

	return $response;
}